/* SPDX-License-Identifier: (GPL-2.0+ OR BSD-2-Clause) */
#ifndef __DT_BINDINGS_CLOCK_MARVELL_MMP2_AUDIO_H
#define __DT_BINDINGS_CLOCK_MARVELL_MMP2_AUDIO_H

#define MMP2_CLK_AUDIO_SYSCLK		0
#define MMP2_CLK_AUDIO_SSPA0		1
#define MMP2_CLK_AUDIO_SSPA1		2

#define MMP2_CLK_AUDIO_NR_CLKS		3
#endif
