/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2016 Neil Armstrong <narmstrong@baylibre.com>
 */

#ifndef DT_CLOCK_OXSEMI_OX810SE_H
#define DT_CLOCK_OXSEMI_OX810SE_H

#define CLK_810_LEON		0
#define CLK_810_DMA_SGDMA	1
#define CLK_810_CIPHER		2
#define CLK_810_SATA		3
#define CLK_810_AUDIO		4
#define CLK_810_USBMPH		5
#define CLK_810_ETHA		6
#define CLK_810_PCIEA		7
#define CLK_810_NAND		8

#endif /* DT_CLOCK_OXSEMI_OX810SE_H */
