/* SPDX-License-Identifier: GPL-2.0-only */
/*
 * Copyright (C) 2020 ARM Ltd.
 */
#ifndef __VDSO_PROCESSOR_H
#define __VDSO_PROCESSOR_H

#ifndef __ASSEMBLY__

#include <asm/vdso/processor.h>

#endif /* __ASSEMBLY__ */

#endif /* __VDSO_PROCESSOR_H */
