/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) 2015 Broadcom Corporation
 */

#define BCM2835_AUX_CLOCK_UART		0
#define BCM2835_AUX_CLOCK_SPI1		1
#define BCM2835_AUX_CLOCK_SPI2		2
#define BCM2835_AUX_CLOCK_COUNT		3
